@@ -5,7 +5,7 @@
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -14,9 +14,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
+
 #ifndef G10_ELGAMAL_H
 #define G10_ELGAMAL_H
 
@@ -25,11 +25,9 @@ int elg_check_secret_key( int algo, MPI *skey );
 int elg_encrypt( int algo, MPI *resarr, MPI data, MPI *pkey );
 int elg_decrypt( int algo, MPI *result, MPI *data, MPI *skey );
 int elg_sign( int algo, MPI *resarr, MPI data, MPI *skey );
-int elg_verify( int algo, MPI hash, MPI *data, MPI *pkey,
-		    int (*cmp)(void *, MPI), void *opaquev );
+int elg_verify( int algo, MPI hash, MPI *data, MPI *pkey );
 unsigned elg_get_nbits( int algo, MPI *pkey );
 const char *elg_get_info( int algo, int *npkey, int *nskey,
 				    int *nenc, int *nsig, int *use );
 
-
 #endif /*G10_ELGAMAL_H*/
